#include <stdio.h>
#include "HV_interface.h"

//
// Hypervisor system call wrappers
//
int HV_get_version_info( u64* v1 )
{
   u64 stat;
   xorhack_syscall( 127, 0,     0,  0,0,0,0,0,0,
 		         &stat, v1, 0,0,0,0,0,0 );
   return (int)stat;
}

int HV_get_logical_partition_id( u64* v1 )
{
   u64 stat;
   xorhack_syscall( 74, 0,     0,  0,0,0,0,0,0,
 		        &stat, v1, 0,0,0,0,0,0 );
   return (int)stat;
}

int HV_get_logical_ppe_id( u64* v1 )
{
   u64 stat;
   xorhack_syscall( 69, 0,     0,  0,0,0,0,0,0,
 		        &stat, v1, 0,0,0,0,0,0 );
   return (int)stat;
}

int HV_get_virtual_adress_space_id_of_ppe( u64 ppe_id, u64* v1 )
{
   u64 stat;
   xorhack_syscall(  4, ppe_id, 0,  0,0,0,0,0,0,
 		        &stat,  v1, 0,0,0,0,0,0 );
   return (int)stat;
}

int HV_map_htab( u64 lpar, u64* v1 )
{
   u64 stat;
   xorhack_syscall(  122, lpar,  0,  0,0,0,0,0,0,
 		          &stat, v1, 0,0,0,0,0,0 );
   return (int)stat;
}

int HV_create_repository_node_value( u64 key1, u64 key2, u64 key3, u64 key4,
                                  u64 v1, u64 v2 )
{
   u64 stat;
   xorhack_syscall( 90, key1,  key2, key3, key4, v1, v2, 0,0,
		        &stat, 0,    0,    0,    0,  0,  0,0 );
   return (int)stat;
}

int HV_get_repository_node_value( u64 p_id, u64 key1, u64 key2, u64 key3, u64 key4, 
                                  u64* v1, u64* v2 )
{
   u64 stat;
   xorhack_syscall( 91, p_id,  key1, key2, key3, key4, 0,0,0,
		        &stat, v1,   v2,   0,    0,    0,0,0 );
   return (int)stat;
}

int HV_modify_repository_node_value( u64 key1, u64 key2, u64 key3, u64 key4,
                                  u64 v1, u64 v2 )
{
   u64 stat;
   xorhack_syscall( 92, key1,  key2, key3, key4, v1, v2, 0,0,
		        &stat, 0,    0,    0,    0,  0,  0,0 );
   return (int)stat;
}

int HV_remove_repository_node_value( u64 key1, u64 key2, u64 key3, u64 key4 )
{
   u64 stat;
   xorhack_syscall( 93, key1,  key2, key3, key4, 0,0,0,0,
		        &stat, 0,    0,    0,    0,0,0,0 );
   return (int)stat;
}

int HV_construct_logical_spe( u64 shift1, u64 shift2, u64 shift3, u64 shift4, u64 shift5, 
                              u64 vas_id, u64 mode,
                              u64* priv2adr, u64* problphys, u64* LS_store, u64* dummy,
                              u64* shadowadr, u64* spe_id )
{
   u64 stat;
   xorhack_syscall( 57, shift1, shift2,    shift3,     shift4,    shift5, vas_id,     0,      0,
		        &stat,  priv2adr,  problphys,  LS_store,  dummy,  shadowadr,  spe_id, 0 );
   return (int)stat;
}

int HV_destruct_logical_spe( u64 spe_id )
{
   u64 stat;
   xorhack_syscall( 69, spe_id, 0,0,0,0,0,0,0,
 		        &stat,  0,0,0,0,0,0,0 );
   return (int)stat;
}

int HV_enable_logical_spe( u64 spe_id, u64 res_id )
{
   u64 stat;
   xorhack_syscall( 207, spe_id, res_id, 0,0,0,0,0,0,
 		        &stat,   0,      0,0,0,0,0,0 );
   return (int)stat;
}

int HV_disable_logical_spe( u64 spe_id, u64 mode )
{
   u64 stat;
   xorhack_syscall( 65, spe_id, mode, 0,0,0,0,0,0,
 		        &stat,  0,    0,0,0,0,0,0 );
   return (int)stat;
}

//
// HV calls common error message display
//
void HV_error( int errno )
{ 
   switch( errno )
   {     
   case HV_Hardware:	printf("HV hardware error\n");
			break;
   case HV_Function: 	printf("HV function not supported\n");
			break;
   case HV_Privilege:	printf("HV permission denied\n");
			break;
   case HV_Parameter:	printf("HV illegal parameter(s)\n");
			break;
   case HV_Access:	printf("HV access violation\n");
			break;
   case HV_No_Entry:	printf("HV no such entry\n");
			break;
   case HV_Duplicate:	printf("HV duplicate entry\n");
			break;
   case HV_Busy:	printf("HV device is busy\n");
			break;
   case HV_Wrong_State:	printf("HV is in wrong state\n");
			break;
   case HV_Illegal_Par:	printf("HV illegal parameter value\n");
			break;
   case HV_No_Implemen:	printf("HV function not implemented\n");
			break;
   default:		printf("HV error %d\n", errno ); 
   }
}

