//
// do nice things about SPU's with the HV
//

#include <stdio.h>
#include <xorhack.h>
#include "HV_interface.h"

#define PS3_PPU_ID			0x0000000000000001
#define PS3_VAS_ID 		        0x0000000000000002

#define PS3_LPAR_ID_LINUX	        0x0000000000000002
#define BI				0x0000000062690000
#define SPUN				0x7370756e00000000
#define SPURSVN				0x7370757273766e00
#define SPURSV				0x7370757273760000
#define PS3_SPU_RESOURCE_TYPE_EXCLUSIVE 0x8000000000000000
#define SPU_TYPE_LOGICAL		0x0000000000000000
#define PAGE_SHIFT			0x0000000000000010

#define PS3_LPAR_ID_PME			0x0000000000000001
#define SYS				0x0000000073797300
#define BE				0x6265000000000000
#define SPURSVSL			0x737075727376736c
#define AUSRSPUN			0x617573727370756e
#define ASYSSPUN			0x617379737370756e		


// list firmware version

int lsfw()
{
   u64 v1, v2;
   u64 status;
   char s[5];  // output version in readable form

   if( xorhack_init() )
   {
	printf("XH init error\n");
	return -1;
   }

   // do the HV call
   status = HV_get_version_info( &v1 );


   if ( status == HV_Success )
   {
      s[3] = (char)( v1      & (unsigned long long)0xf) | 0x30;
      s[2] = (char)((v1>>16) & (unsigned long long)0xf) | 0x30;
      s[1] = 0x2e;
      s[0] = (char)((v1>>32) & (unsigned long long)0xf) | 0x30;
      s[4] = 0; 
      printf("PS3 firmware %s\n", s );
   }
   else
      HV_error( status );
   
   if( xorhack_free() )
   {
       printf("XH free error\n");
       return -1;
   }

   return 0;
}

// list available sys.be info
int lssys()
{
   u64 v1, v2;
   u64 status;

   if( xorhack_init() )
   {
	printf("XH init error\n");
	return -1;
   }

   // do HV call
   status = HV_get_repository_node_value( PS3_LPAR_ID_PME, SYS, BE, 0, SPURSVSL, &v1, &v2);

   if ( status == HV_Success )
   {
      printf("SPU reservation key  :  %#llx\n", v1 );
   }
   else
      HV_error( status );

   // do HV call
   status = HV_get_repository_node_value( PS3_LPAR_ID_PME, SYS, BE, 0, AUSRSPUN, &v1, &v2);

   if ( status == HV_Success )
   {
      printf("Number of user SPUs  :  %#llx\n", v1 );
   }
   else
      HV_error( status );

   // do HV call
   status = HV_get_repository_node_value( PS3_LPAR_ID_PME, SYS, BE, 0, ASYSSPUN, &v1, &v2);

   if ( status == HV_Success )
   {
      printf("Number of system SPUs:  %#llx\n", v1 );
   }
   else
      HV_error( status );
   
   if( xorhack_free() )
   {
       printf("XH free error\n");
       return -1;
   }

   return 0;
}

// list available SPUs
int lsspu()
{
   u64 v1, v2;
   u64 n_spus, n_res_spus, mode, id;
   u64 status;

   if( xorhack_init() )
   {
	printf("XH init error\n");
	return -1;
   }

   // do HV call
   status = HV_get_repository_node_value( PS3_LPAR_ID_LINUX, BI, SPUN, 0, 0, &n_spus, &v2);

   if ( status == HV_Success )
   {
      printf("Number of total SPUs %llu\n", n_spus );
   }
   else
      HV_error( status );

   // do HV call
   status = HV_get_repository_node_value( PS3_LPAR_ID_LINUX, BI, SPURSVN, 0, 0, &n_res_spus, &v2);

   if ( status == HV_Success )
   {
      printf("Number of reserved SPUs %llu\n", n_res_spus );
   }
   else
      HV_error( status );

   for ( id = 0; id < n_res_spus; id++ )
   {
      // do HV call
      status = HV_get_repository_node_value( PS3_LPAR_ID_LINUX, BI, SPURSV, id, 0, &mode, &v2);

      printf("%llu: ", id+1 );
      if ( status == HV_Success )
      {
         printf("SPU %llu ", v2 );
         if (mode == PS3_SPU_RESOURCE_TYPE_EXCLUSIVE) printf("exclusive");
         else                                         printf("unknown");
         printf("\n");
      }
      else
         HV_error( status );
   }
   
   if( xorhack_free() )
   {
       printf("XH free error\n");
       return -1;
   }

   return 0;
}

// increase number in HV table
int incspu()
{
   u64 v1, v2;
   u64 n_spus, n_res_spus, mode, id;
   u64 priv2adr, problphys, LS_store, dummy, shadowadr, spe_id;  
   u64 status;

   if( xorhack_init() )
   {
	printf("XH init error\n");
	return -1;
   }

   // get number of current
   status = HV_get_repository_node_value( PS3_LPAR_ID_LINUX, BI, SPUN, 0, 0, &n_spus, &v2);
   if ( !status ) status = HV_get_repository_node_value( PS3_LPAR_ID_LINUX, BI, SPURSVN, 0, 0,
                                                         &n_res_spus, &v2);
   if ( status != HV_Success ) 
   { 
      HV_error( status );
      return -1;
   }

   // increment
   n_spus++;
   n_res_spus++;

   // do HV call
   status = HV_modify_repository_node_value( BI, SPUN, 0, 0, n_spus, 0);
   if ( !status ) status = HV_modify_repository_node_value( BI, SPURSVN, 0, 0, n_res_spus, 0);
   if ( !status ) status = HV_create_repository_node_value( BI, SPURSV, n_res_spus-1, 0, 
					       PS3_SPU_RESOURCE_TYPE_EXCLUSIVE, n_res_spus-1);
   if ( status == HV_Success )
   {
      printf("Number of SPUs in HV partition increased\n" );
   }
   else
      HV_error( status );

   status = HV_construct_logical_spe( PAGE_SHIFT, PAGE_SHIFT, PAGE_SHIFT, PAGE_SHIFT, PAGE_SHIFT, 
                              PS3_VAS_ID, SPU_TYPE_LOGICAL,
                              &priv2adr, &problphys, &LS_store, &dummy,
                              &shadowadr, &spe_id );
   if ( status == HV_Success )
   {
      printf("Created SPU %lld: id %#llx, priv=%#llx, phys=%#llx, LS=%#llx, shadow=%#llx\n",
                       n_res_spus-1, spe_id, priv2adr, problphys, LS_store, shadowadr );
   }
   else
      HV_error( status );

   status = HV_enable_logical_spe( spe_id, n_res_spus-1 );
   if ( status == HV_Success )
   {
      printf("Enabled SPU %lld\n", n_res_spus-1 );
   }
   else
      HV_error( status );

   if( xorhack_free() )
   {
       printf("XH free error\n");
       return -1;
   }

   return 0;
}

// decrease number in HV table
int decspu()
{
   u64 v1, v2;
   u64 n_spus, n_res_spus, mode, id;
   u64 status;

   if( xorhack_init() )
   {
	printf("XH init error\n");
	return -1;
   }

   // get number of current
   status = HV_get_repository_node_value( PS3_LPAR_ID_LINUX, BI, SPUN, 0, 0, &n_spus, &v2);
   if ( status != HV_Success ) 
   { 
      HV_error( status );
      return -1;
   }
   status = HV_get_repository_node_value( PS3_LPAR_ID_LINUX, BI, SPURSVN, 0, 0, &n_res_spus, &v2);
   if ( status != HV_Success ) 
   { 
      HV_error( status );
      return -1;
   }

   // check
   if ( n_spus == 0 || n_res_spus == 0 ) 
   {
      printf("Already 0\n");
      return -1;
   }

   // decrement
   n_spus--;
   n_res_spus--;

   // do HV call
   status = HV_modify_repository_node_value( BI, SPUN, 0, 0, n_spus, 0);
   if ( !status ) status = HV_modify_repository_node_value( BI, SPURSVN, 0, 0, n_res_spus, 0);
   if ( !status ) status = HV_remove_repository_node_value( BI, SPURSV, n_res_spus, 0 ); 

   if ( status == HV_Success )
   {
      printf("Number of SPUs in HV partition decreased\n" );
   }
   else
      HV_error( status );
   
   if( xorhack_free() )
   {
       printf("XH free error\n");
       return -1;
   }

   return 0;
}

// remove all entries
int rmspu()
{
   u64 v1, v2;
   u64 n_res_spus, mode, id;
   u64 status;

   if( xorhack_init() )
   {
	printf("XH init error\n");
	return -1;
   }

    // SPU eraser
    for ( id = 0; id < 64; id++ )
    {
      // do HV call
      status = HV_disable_logical_spe( id, 0 );
      
      if ( status == HV_Success )
      { 
         printf("%#llx disabled", id );

         status = HV_destruct_logical_spe( id );
         if ( status == HV_Success )
         {
            printf(" & destroyed");
         }
         printf("\n");
      }
   }

   // do HV call 
   status = HV_modify_repository_node_value( BI, SPURSVN, 0, 0, 0, 0); 
   if ( !status ) status = HV_modify_repository_node_value( BI, SPUN, 0, 0, 0, 0);

   if ( status == HV_Success )
   {
      printf("SPU number reset to 0\n" );
   }
   else
      HV_error( status );

   for ( id = 0; id < n_res_spus; id++ )
   {
      // do HV call
      status = HV_remove_repository_node_value( BI, SPURSV, id, 0 ); 
      if ( status == HV_Success )
      {
         printf("Removed node %llu \n", id );
      }
      else
         HV_error( status );
    }
   
   if( xorhack_free() )
   {
       printf("XH free error\n");
       return -1;
   }

   return 0;
}

// list available PPU
int lsppu()
{
   u64 v1, v2;
   u64 lpar, vas_id, ppu_id;
   u64 status;

   if( xorhack_init() )
   {
	printf("XH init error\n");
	return -1;
   }

   // do HV call
   status = HV_get_logical_ppe_id( &ppu_id );

   if ( status == HV_Success )
   {
      printf("PPU id: %llu\n", ppu_id );
   }
   else
      HV_error( status );

   // do HV call
   status = HV_get_logical_partition_id( &lpar );

   if ( status == HV_Success )
   {
      printf("Logical partition id: %llu\n", lpar );
   }
   else
      HV_error( status );

   // do HV call
   status = HV_get_virtual_adress_space_id_of_ppe( ppu_id, &vas_id );

   if ( status == HV_Success )
   {
      printf("Virtual adress space id of PPU %llu: %llu\n", ppu_id, vas_id );
   }
   else
      HV_error( status );
   
   if( xorhack_free() )
   {
       printf("XH free error\n");
       return -1;
   }

   return 0;
}

// print usage message
void pr_usage( char * arg0 )
{
   printf("\n");
   printf("Usage:  '%s OPTION'\n", arg0 );
   printf("Options:\n");
   printf("       -f    shows firmware version\n");
   printf("       -l    lists available SPUs\n");
   printf("       -i    increases repo number by 1, try to create SPU\n");
   printf("       -d    decreases repo number by 1\n");
   printf("       -x    erase each possible SPU (useful to find SPU ids; dangerous)\n");
   printf("       -p    lists PPU entries\n");
   printf("       -s    lists sys.be SPU entries\n");
}


//
// main 
//
int main( int argc, char **argv ) 
{
   // Parse command line arg
   if (argc != 2) 
   {
      pr_usage( argv[0] );
      return 1;
   }
   else if (argv[1][0] != '-') 
   {
      pr_usage( argv[0] );
      return 1;
   }
   else 
      // call function subroutines
      switch( argv[1][1] )
      {
         case 'l':	lsspu();
			break;
         case 'i':	incspu();
			break;
         case 'd':	decspu();
			break;
         case 'x':	rmspu();
			break;
         case 'p':	lsppu();
			break;
         case 's':	lssys();
			break;
         case 'f':	lsfw();
			break;
         default:	pr_usage( argv[0] );
      }

   return 0;
}

 
